-- luacheck: globals app connect
local Class = require "Base.Class"
local Unit = require "Unit"
local Pitch = require "Unit.ViewControl.Pitch"
local GainBias = require "Unit.ViewControl.GainBias"
local Encoder = require "Encoder"

local FMOperator = Class{}
FMOperator:include(Unit)

function FMOperator:init(args)
  args.title = "FM Op"
  args.mnemonic = "FM"
  args.version = 2
  Unit.init(self,args)
end

function FMOperator:onLoadGraph(channelCount)
  local carrier = self:createObject("SineOscillator","carrier")
  local modulator = self:createObject("SineOscillator","modulator")
  local rational = self:createObject("RationalMultiply","rational",true)
  local multiply = self:createObject("Multiply","multiply")
  local tune = self:createObject("ConstantOffset","tune")
  local tuneRange = self:createObject("MinMax","tuneRange")
  local f0 = self:createObject("GainBias","f0")
  local f0Range = self:createObject("MinMax","f0Range")
  local vca = self:createObject("Multiply","vca")
  local level = self:createObject("GainBias","level")
  local levelRange = self:createObject("MinMax","levelRange")
  local num = self:createObject("GainBias","num")
  local numRange = self:createObject("MinMax","numRange")
  local den = self:createObject("GainBias","den")
  local denRange = self:createObject("MinMax","denRange")
  local index = self:createObject("GainBias","index")
  local indexRange = self:createObject("MinMax","indexRange")

  connect(tune,"Out",tuneRange,"In")
  connect(tune,"Out",carrier,"V/Oct")

  connect(f0,"Out",carrier,"Fundamental")
  connect(f0,"Out",f0Range,"In")

  connect(f0,"Out",rational,"In")
  connect(rational,"Out",modulator,"Fundamental")
  connect(num,"Out",rational,"Numerator")
  connect(num,"Out",numRange,"In")
  connect(den,"Out",rational,"Divisor")
  connect(den,"Out",denRange,"In")

  connect(index,"Out",multiply,"Left")
  connect(index,"Out",indexRange,"In")
  connect(modulator,"Out",multiply,"Right")
  connect(multiply,"Out",carrier,"Phase")

  connect(level,"Out",levelRange,"In")
  connect(level,"Out",vca,"Left")

  connect(carrier,"Out",vca,"Right")
  connect(vca,"Out",self,"Out1")

  self:createMonoBranch("level",level,"In",level,"Out")
  self:createMonoBranch("tune",tune,"In",tune,"Out")
  self:createMonoBranch("index",index,"In",index,"Out")
  self:createMonoBranch("f0",f0,"In",f0,"Out")
  self:createMonoBranch("num",num,"In",num,"Out")
  self:createMonoBranch("den",den,"In",den,"Out")

  if channelCount > 1 then
    connect(self.objects.vca,"Out",self,"Out2")
  end
end

local views = {
  expanded = {"tune","freq","num","den","index","level"},
  collapsed = {},
}

function FMOperator:onLoadViews(objects,branches)
  local controls = {}

  controls.tune = Pitch {
    button = "V/oct",
    branch = branches.tune,
    description = "V/oct",
    offset = objects.tune,
    range = objects.tuneRange
  }

  controls.freq = GainBias {
    button = "f0",
    description = "Fundamental",
    branch = branches.f0,
    gainbias = objects.f0,
    range = objects.f0Range,
    biasMap = Encoder.getMap("oscFreq"),
    biasUnits = app.unitHertz,
    initialBias = 27.5,
    gainMap = Encoder.getMap("freqGain"),
    scaling = app.octaveScaling
  }

  controls.num = GainBias {
    button = "num",
    description = "Numerator",
    branch = branches.num,
    gainbias = objects.num,
    range = objects.numRange,
    biasMap = Encoder.getMap("int[1,32]"),
    biasPrecision = 0,
    initialBias = 1
  }

  controls.den = GainBias {
    button = "den",
    description = "Denominator",
    branch = branches.den,
    gainbias = objects.den,
    range = objects.denRange,
    biasMap = Encoder.getMap("int[1,32]"),
    biasPrecision = 0,
    initialBias = 1
  }

  controls.index = GainBias {
    button = "index",
    description = "Index",
    branch = branches.index,
    gainbias = objects.index,
    range = objects.indexRange,
    initialBias = 0.1,
  }

  controls.level = GainBias {
    button = "level",
    description = "Level",
    branch = branches.level,
    gainbias = objects.level,
    range = objects.levelRange,
    initialBias = 0.5,
  }

  return controls, views
end

return FMOperator
